package com.freemarker;

import freemarker.template.Configuration;
import freemarker.template.DefaultObjectWrapper;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class TemplateApplicator {
	private static final Logger log = LoggerFactory
			.getLogger(TemplateApplicator.class);

	public static Configuration cfg = new Configuration();

	public static String applyTemplate(String templateName,
			Map<String, Object> parameters) {
		try {
			Template temp = cfg.getTemplate(templateName);
			Writer out = new StringWriter();
			temp.process(parameters, out);
			out.flush();
			return out.toString();
		} catch (IOException e) {
			log.error("Cannot find template {} {}", templateName, e);
			System.err.println(e);
			return null;
		} catch (TemplateException e) {
			log.error("Error processing template {}: {}", templateName, e);
			System.err.println(e);
		}
		return null;
	}

	static {
		try {
			cfg.setClassForTemplateLoading(TemplateApplicator.class, "/");

			cfg.setObjectWrapper(new DefaultObjectWrapper());
			cfg.setOutputEncoding("UTF-8");
			cfg.setURLEscapingCharset(null);
		} catch (Exception e) {
			log.error("Cannot initialize Freemarker engine", e);
		}
	}
}