package org.serviio.restui;

import org.restlet.Application;
import org.restlet.Component;
import org.restlet.Context;
import org.restlet.Restlet;
import org.restlet.Server;
import org.restlet.data.Protocol;
import org.restlet.engine.application.Encoder;
import org.restlet.resource.Directory;
import org.restlet.routing.Extractor;
import org.restlet.routing.Redirector;
import org.restlet.routing.Router;
import org.serviio.cache.CachedDirectory;
import org.serviio.restui.template.TemplatedConsole;
import org.serviio.restui.template.TemplatedDirectory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class RestletProxy {
	private static final Logger log = LoggerFactory
			.getLogger(RestletProxy.class);

	private final static int PORT = 8123;

	private static Component component = new Component();

	public static void runServer() {
		String remoteHost = System.getProperty("serviio.remoteHost");
		if (remoteHost == null || "".equals(remoteHost)) {
			remoteHost = "localhost";
		}
		log.info("Starting Restlet proxy exposed on {}:{}", remoteHost, PORT);

		// Configure servers & clients
		component.getServers().add(Protocol.HTTP, remoteHost, PORT);
		component.getClients().add(Protocol.HTTP);
		component.getClients().add(Protocol.FILE);

		// hack for having more than 10 connections
		// Consider to upgrade to NIO Restlet...
		Server server1 = component.getServers().get(0);
		server1.getContext().getParameters().add("maxTotalConnections", "30");
		server1.getContext().getParameters().add("maxThreads", "30");

		Application app = new Application(component.getContext()
				.createChildContext()) {
			static final String STATIQUE_URI = "file:../web/statique";
			static final String ROOT_URI = "file:/";
			static final String target = "http://localhost:23423/rest/{keywords}";

			public Restlet createInboundRoot() {
				Context context = getContext();
				Router router = new Router(context);

				Redirector redirector = new Redirector(context, target,
						Redirector.MODE_SERVER_OUTBOUND);

				// http://localhost:8182/search?kwd=myKeyword1+myKeyword2
				// =>
				// http://www.google.com/search?q=site:mysite.org+myKeyword1%20myKeyword2
				Extractor extractor = new Extractor(context, redirector);
				extractor.extractFromQuery("keywords", "k", true);
				router.attach("/proxy", extractor);

				// console
				router.attach("/console/{language}", TemplatedConsole.class);
				router.attach("/console", TemplatedConsole.class);

				// Static dir
				Directory dir = new CachedDirectory(context, STATIQUE_URI);
				dir.setListingAllowed(true);
				router.attach("/statique", dir);

				// Navigation
				Directory dir2 = new TemplatedDirectory(context, ROOT_URI);
				dir2.setListingAllowed(true);
				router.attach("/dir", dir2);

				// Encoder
				Encoder encoder = new Encoder(context);
				encoder.setEncodingRequest(true);
				encoder.setEncodingResponse(true);
				encoder.setNext(router);

				return encoder;
			}
		};

		try {
			component.getDefaultHost().attach("/serviio", app);
			component.start();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static void stopServer() {
		try {
			component.stop();
		} catch (Exception e) {
			log.warn("Error during shutting down Restlet server", e);
		}
	}

	public static void main(String[] args) {
		runServer();
	}

}