package org.serviio.restui.template;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.restlet.Context;
import org.restlet.data.MediaType;
import org.restlet.data.Reference;
import org.restlet.data.ReferenceList;
import org.restlet.representation.Representation;
import org.restlet.representation.Variant;
import org.restlet.resource.Directory;

import com.freemarker.TemplateApplicator;
import com.freemarker.TemplateRepresentation;

public class TemplatedDirectory extends Directory {

	/** Test if Windows env. */
	private static boolean isWindows = ('\\' == File.separatorChar);

	/** Template directory */
	public final static String TPL_DIR = "org/serviio/restui/template/";

	public TemplatedDirectory(Context context, String rootUri) {
		this(context, new Reference(rootUri));
	}

	public TemplatedDirectory(Context context, Reference rootLocalReference) {
		super(context, rootLocalReference);
	}

	/**
	 * Html representation only.
	 */
	public List<Variant> getIndexVariants(ReferenceList indexContent) {
		return Arrays.asList(new Variant(MediaType.TEXT_HTML));
	}

	/**
	 * Select representations.
	 */
	public Representation getIndexRepresentation(Variant variant,
			ReferenceList indexContent) {
		if (variant.getMediaType().equals(MediaType.TEXT_HTML))
			return getWebRepresentationTpl(indexContent);
		return super.getIndexRepresentation(variant, indexContent);
	}

	/**
	 * Freemarker HTML representation.
	 * 
	 * @param indexContent
	 * @return the HTML representation.
	 */
	public Representation getWebRepresentationTpl(ReferenceList indexContent) {
		Reference id = indexContent.getIdentifier();
		List<String> segments = id.getSegments();

		// File roots
		List<String> rootList = new ArrayList<String>();
		for (File f : File.listRoots()) {
			if (isWindows)
				rootList.add(f.getPath().replace(File.separator, ""));
			else
				rootList.add(f.getPath());
		}

		// Add file root if missing
		if (isWindows && !rootList.contains(segments.get(2))) {
			String root = new File("/").getAbsolutePath().replace(
					File.separator, "");
			segments.add(2, root);
			id.setSegments(segments);
		}

		// compute relative and absolute urls
		String rootRef = String.format("/%s/%s/", segments.get(0),
				segments.get(1));
		String relativeRef = id.getPath(true).replaceFirst(rootRef, "");

		// dataModel
		Map<String, Object> dataModel = new HashMap<String, Object>();
		dataModel.put("indexContent", indexContent);
		dataModel.put("identifier", id);
		dataModel.put("roots", rootList);
		dataModel.put("rootRef", rootRef);
		dataModel.put("relRef", relativeRef);

		TemplateRepresentation representation = new TemplateRepresentation(
				TPL_DIR + "directory.tpl.jsp", TemplateApplicator.cfg, dataModel,
				MediaType.TEXT_HTML);
		return representation;
	}

}